@@ -905,6 +905,73 @@ TEST_CASE_METHOD(
   }
 }
 
+TEST_CASE_METHOD(
+    SerializationFx,
+    "Query serialization, sparse, old client",
+    "[query][sparse][serialization][old-client]") {
+  create_array(TILEDB_SPARSE);
+  write_sparse_array();
+
+  Config config;
+  config.set("sm.query.sparse_global_order.reader", "legacy");
+  config.set("sm.query.sparse_unordered_with_dups.reader", "legacy");
+  auto ctx_client = Context(config);
+
+  SECTION("- Read all") {
+    Array array(ctx_client, array_uri, TILEDB_READ);
+    Query query(ctx_client, array);
+    std::vector<int32_t> coords(1000);
+    std::vector<uint32_t> a1(1000);
+    std::vector<uint32_t> a2(1000);
+    std::vector<uint8_t> a2_nullable(1000);
+    std::vector<char> a3_data(1000 * 100);
+    std::vector<uint64_t> a3_offsets(1000);
+    std::vector<int32_t> subarray = {1, 10, 1, 10};
+
+    query.set_layout(TILEDB_GLOBAL_ORDER);
+    query.set_subarray(subarray);
+    query.set_coordinates(coords);
+    query.set_data_buffer("a1", a1);
+    query.set_data_buffer("a2", a2);
+    query.set_validity_buffer("a2", a2_nullable);
+    query.set_data_buffer("a3", a3_data);
+    query.set_offsets_buffer("a3", a3_offsets);
+
+    // Serialize into a copy and submit.
+    std::vector<uint8_t> serialized;
+    serialize_query(ctx_client, query, &serialized, true);
+
+    Array array2(ctx, array_uri, TILEDB_READ);
+    Query query2(ctx, array2);
+    deserialize_query(ctx, serialized, &query2, false);
+    auto to_free = allocate_query_buffers(ctx, array2, &query2);
+    query2.submit();
+    serialize_query(ctx, query2, &serialized, false);
+
+    // Make sure query2 has logged stats
+    check_read_stats(query2);
+
+    // Deserialize into original query
+    deserialize_query(ctx_client, serialized, &query, true);
+    REQUIRE(query.query_status() == Query::Status::COMPLETE);
+
+    // The deserialized query should also include the write stats
+    check_read_stats(query);
+
+    auto result_el = query.result_buffer_elements_nullable();
+    REQUIRE(std::get<1>(result_el["a1"]) == 10);
+    REQUIRE(std::get<1>(result_el["a2"]) == 20);
+    REQUIRE(std::get<2>(result_el["a2"]) == 10);
+    REQUIRE(std::get<0>(result_el["a3"]) == 10);
+    REQUIRE(std::get<1>(result_el["a3"]) == 55);
+
+    // TODO: check results
+
+    for (void* b : to_free)
+      std::free(b);
+  }
+}
+
 TEST_CASE_METHOD(
     SerializationFx,
     "Query serialization, split coords, sparse",
