@@ -291,3 +291,56 @@ TEST_CASE(
   if (vfs.is_dir(array_name))
     vfs.remove_dir(array_name);
 }
+
+/**
+ * #TODO: Change data_w to be type std::vector<bool>.
+ *
+ * Note: This is not a trivial change; It will require bitset optimization in
+ * the implementation of Query::set_data_buffer.
+ */
+TEST_CASE(
+    "C++ API: Test boolean query buffer", "[cppapi][query][bool_buffer]") {
+  const std::string array_name = "bool_buffer_array";
+  Context ctx;
+  VFS vfs(ctx);
+
+  if (vfs.is_dir(array_name))
+    vfs.remove_dir(array_name);
+
+  // Create the array
+  Domain domain(ctx);
+  domain.add_dimension(Dimension::create<uint8_t>(ctx, "rows", {{0, 3}}, 4))
+      .add_dimension(Dimension::create<uint8_t>(ctx, "cols", {{0, 3}}, 4));
+  ArraySchema schema(ctx, TILEDB_SPARSE);
+  schema.set_domain(domain).set_order({{TILEDB_ROW_MAJOR, TILEDB_ROW_MAJOR}});
+  schema.add_attribute(Attribute::create<uint8_t>(ctx, "a"));
+  Array::create(array_name, schema);
+
+  // Write some initial data and close the array.
+  std::vector<uint8_t> data_w = {0, 1, 0, 1};
+  std::vector<uint8_t> coords_w = {0, 0, 1, 1, 2, 2, 3, 3};
+  Array array_w(ctx, array_name, TILEDB_WRITE);
+  Query query_w(ctx, array_w);
+  query_w.set_coordinates(coords_w)
+      .set_layout(TILEDB_UNORDERED)
+      .set_data_buffer("a", data_w);
+  query_w.submit();
+  query_w.finalize();
+  array_w.close();
+
+  // Read the array.
+  Array array_r(ctx, array_name, TILEDB_READ);
+  Query query_r(ctx, array_r);
+  std::vector<uint8_t> data_r(4);
+  query_r.set_layout(TILEDB_ROW_MAJOR).set_data_buffer("a", data_r);
+  query_r.submit();
+  REQUIRE(data_r[0] == data_w[0]);
+  REQUIRE(data_r[1] == data_w[1]);
+  REQUIRE(data_r[2] == data_w[2]);
+  REQUIRE(data_r[3] == data_w[3]);
+  query_r.finalize();
+  array_r.close();
+
+  if (vfs.is_dir(array_name))
+    vfs.remove_dir(array_name);
+}
