@@ -116,3 +116,47 @@ TEST_CASE(
   if (vfs.is_dir(array_name))
     vfs.remove_dir(array_name);
 }
+
+TEST_CASE(
+    "C++ API updates: empty second write", "[updates][updates-empty-write]") {
+  const std::string array_name = "updates_empty_write";
+  Context ctx;
+  VFS vfs(ctx);
+
+  if (vfs.is_dir(array_name))
+    vfs.remove_dir(array_name);
+
+  // Create
+  Domain domain(ctx);
+  domain.add_dimension(
+      Dimension::create(ctx, "d", TILEDB_STRING_ASCII, nullptr, nullptr));
+  ArraySchema schema(ctx, TILEDB_SPARSE);
+  schema.set_domain(domain).set_order({{TILEDB_ROW_MAJOR, TILEDB_ROW_MAJOR}});
+  Array::create(array_name, schema);
+
+  tiledb_layout_t layout = GENERATE(TILEDB_UNORDERED, TILEDB_GLOBAL_ORDER);
+
+  // First write
+  std::string data("ab");
+  std::vector<uint64_t> offsets({0, 1});
+  Array array_w1(ctx, array_name, TILEDB_WRITE);
+  Query query_w1(ctx, array_w1);
+  query_w1.set_layout(layout).set_data_buffer("d", data).set_offsets_buffer(
+      "d", offsets);
+  query_w1.submit();
+  query_w1.finalize();
+  array_w1.close();
+
+  // Second write
+  offsets.clear();
+  Array array_w2(ctx, array_name, TILEDB_WRITE);
+  Query query_w2(ctx, array_w2);
+  query_w2.set_layout(layout).set_data_buffer("d", data).set_offsets_buffer(
+      "d", offsets);
+  query_w2.submit();
+  query_w2.finalize();
+  array_w2.close();
+
+  if (vfs.is_dir(array_name))
+    vfs.remove_dir(array_name);
+}
