@@ -40,6 +40,7 @@
 #include <catch.hpp>
 
 using namespace tiledb::sm;
+using Catch::Matchers::StartsWith;
 
 TEST_CASE("Buffer: Test resource pool", "[resource-pool]") {
   ResourcePool<int> pool(3);
@@ -56,7 +57,8 @@ TEST_CASE("Buffer: Test resource pool", "[resource-pool]") {
 
     // Try to get one more resource should throw an exception.
     REQUIRE_THROWS_WITH(
-        ResourceGuard(pool), "Ran out of resources in resource pool");
+        ResourceGuard(pool),
+        StartsWith("Ran out of resources in resource pool"));
   }
 
   {
@@ -175,4 +177,4 @@ TEST_CASE("Buffer: Test blocking resource pool possible deadlock", "[.]") {
   REQUIRE((r4.get() == 11 || r4.get() == 10));
   auto other_thread_value = (r4.get() == 11) ? 10 : 11;
   REQUIRE(r5.get() == other_thread_value);
-}
\ No newline at end of file
+}
