@@ -474,6 +474,10 @@ inline Status Status_ArrayDirectoryError(const std::string& msg) {
 inline Status Status_TaskError(const std::string& msg) {
   return {"[TileDB::Task] Error", msg};
 }
+/** Return a Range error class Status with a given message **/
+inline Status Status_RangeError(const std::string& msg) {
+  return {"[TileDB::Range] Error", msg};
+}
 }  // namespace common
 }  // namespace tiledb
 
