@@ -100,6 +100,13 @@ class Array {
     return array_schemas_all_;
   }
 
+  /**
+   * Sets all array schemas.
+   * @param all_schemas The array schemas to set.
+   */
+  void set_array_schemas_all(
+      std::unordered_map<std::string, shared_ptr<ArraySchema>>& all_schemas);
+
   /** Returns the array URI. */
   const URI& array_uri() const;
 
@@ -353,18 +360,55 @@ class Array {
    */
   Metadata* unsafe_metadata();
 
+  /** Set if array metadata is loaded already for this array or not */
+  inline void set_metadata_loaded(const bool is_loaded) {
+    metadata_loaded_ = is_loaded;
+  }
+
+  /** Check if array metadata is loaded already for this array or not */
+  inline bool& metadata_loaded() {
+    return metadata_loaded_;
+  }
+
+  /** Check if non emtpy domain is loaded already for this array or not */
+  inline bool& non_empty_domain_computed() {
+    return non_empty_domain_computed_;
+  }
+
   /** Returns the non-empty domain of the opened array.
    *  If the non_empty_domain has not been computed or loaded
    *  it will be loaded first
    * */
   tuple<Status, optional<const NDRange>> non_empty_domain();
 
+  /**
+   * Retrieves the array metadata object that is already loadad.
+   * If it's not yet loaded it will be empty.
+   */
+  NDRange* loaded_non_empty_domain();
+
   /** Returns the non-empty domain of the opened array. */
   void set_non_empty_domain(const NDRange& non_empty_domain);
 
+  /** Set if the non_empty_domain is computed already for this array or not */
+  inline void set_non_empty_domain_computed(const bool is_computed) {
+    non_empty_domain_computed_ = is_computed;
+  }
+
   /** Returns the memory tracker. */
   MemoryTracker* memory_tracker();
 
+  /** Checks the config to see if non empty domain should be serialized on array
+   * open. */
+  bool serialize_non_empty_domain() const;
+
+  /** Checks the config to see if metadata should be serialized on array open.
+   */
+  bool serialize_metadata() const;
+
+  /** Checks the config to see if refactored array open should be used. */
+  bool use_refactored_array_open() const;
+
  private:
   /* ********************************* */
   /*         PRIVATE ATTRIBUTES        */
