@@ -85,6 +85,7 @@ class FragmentMetadata {
    *     1970-01-01 00:00:00 +0000 (UTC).
    * @param dense Indicates whether the fragment is dense or sparse.
    * @param has_timestamps Does the fragment contains timestamps.
+   * @param has_delete_meta Does the fragment contains delete metadata.
    */
   FragmentMetadata(
       StorageManager* storage_manager,
@@ -93,7 +94,8 @@ class FragmentMetadata {
       const URI& fragment_uri,
       const std::pair<uint64_t, uint64_t>& timestamp_range,
       bool dense = true,
-      bool has_timestamps = false);
+      bool has_timestamps = false,
+      bool has_delete_mata = false);
 
   /** Destructor. */
   ~FragmentMetadata();
@@ -238,6 +240,11 @@ class FragmentMetadata {
     return has_timestamps_;
   }
 
+  /** Returns true if the fragment has delete metadata. */
+  inline bool has_delete_meta() const {
+    return has_delete_meta_;
+  }
+
   /**
    * Retrieves the overlap of all MBRs with the input ND range.
    */
@@ -296,7 +303,14 @@ class FragmentMetadata {
    *
    * Applicable to format versions 12 or higher.
    */
-  Status store_v12_or_higher(const EncryptionKey& encryption_key);
+  Status store_v12_v14(const EncryptionKey& encryption_key);
+
+  /**
+   * Stores all the metadata to storage.
+   *
+   * Applicable to format versions 15 or higher.
+   */
+  Status store_v15_or_higher(const EncryptionKey& encryption_key);
 
   /** Returns the non-empty domain in which the fragment is constrained. */
   const NDRange& non_empty_domain();
@@ -391,11 +405,9 @@ class FragmentMetadata {
    * @param name The attribute for which the min is set.
    * @param tid The index of the tile for which the min is set.
    * @param min The minimum.
-   * @param size The size.
    * @return void
    */
-  void set_tile_min(
-      const std::string& name, uint64_t tid, const void* min, uint64_t size);
+  void set_tile_min(const std::string& name, uint64_t tid, const ByteVec& min);
 
   /**
    * Sets a tile min size for the var input attribute.
@@ -416,7 +428,8 @@ class FragmentMetadata {
    * @param min The minimum.
    * @return void
    */
-  void set_tile_min_var(const std::string& name, uint64_t tid, const void* min);
+  void set_tile_min_var(
+      const std::string& name, uint64_t tid, const ByteVec& min);
 
   /**
    * Sets a tile max for the input attribute.
@@ -424,11 +437,9 @@ class FragmentMetadata {
    * @param name The attribute for which the max is set.
    * @param tid The index of the tile for which the max is set.
    * @param max The maximum.
-   * @param size The size.
    * @return void
    */
-  void set_tile_max(
-      const std::string& name, uint64_t tid, const void* max, uint64_t size);
+  void set_tile_max(const std::string& name, uint64_t tid, const ByteVec& max);
 
   /**
    * Sets a tile max for the var input attribute.
@@ -449,7 +460,8 @@ class FragmentMetadata {
    * @param max The maximum.
    * @return void
    */
-  void set_tile_max_var(const std::string& name, uint64_t tid, const void* max);
+  void set_tile_max_var(
+      const std::string& name, uint64_t tid, const ByteVec& max);
 
   /**
    * Converts min/max sizes to offsets.
@@ -467,7 +479,7 @@ class FragmentMetadata {
    * @param sum The sum.
    * @return void
    */
-  void set_tile_sum(const std::string& name, uint64_t tid, const ByteVec* sum);
+  void set_tile_sum(const std::string& name, uint64_t tid, const ByteVec& sum);
 
   /**
    * Sets a tile null count for the input attribute.
@@ -700,6 +712,15 @@ class FragmentMetadata {
    */
   tuple<Status, optional<uint64_t>> get_null_count(const std::string& name);
 
+  /**
+   * Retrieves the processed conditions. The processed conditions is the list
+   * of delete/update conditions that have already been applied for this
+   * fragment and don't need to be applied again.
+   *
+   * @return Processed conditions.
+   */
+  std::unordered_set<std::string>& get_processed_conditions();
+
   /** Returns the first timestamp of the fragment timestamp range. */
   uint64_t first_timestamp() const;
 
@@ -787,26 +808,37 @@ class FragmentMetadata {
   Status load_fragment_min_max_sum_null_count(
       const EncryptionKey& encryption_key);
 
+  /**
+   * Loads the processed conditions for the fragment. The processed conditions
+   * is the list of delete/update conditions that have already been applied for
+   * this fragment and don't need to be applied again.
+   *
+   * @param encryption_key The key the array got opened with.
+   * @return Status
+   */
+  Status load_processed_conditions(const EncryptionKey& encryption_key);
+
   /**
    * Checks if the fragment overlaps partially (not fully) with a given
-   * array open - end time.
+   * array open - end time. Assumes overlapping fragment and array open - close
+   * times.
    *
    * @param array_start_timestamp Array open time
    * @param array_end_timestamp Array end time
    *
-   * @return True if there is partial overlap, false if there is full or no
-   * overlap
+   * @return True if there is partial overlap, false if there is full
    */
   inline bool partial_time_overlap(
       const uint64_t array_start_timestamp,
       const uint64_t array_end_timestamp) const {
     const auto fragment_timestamp_start = timestamp_range_.first;
     const auto fragment_timestamp_end = timestamp_range_.second;
-
-    return (array_start_timestamp > fragment_timestamp_start &&
-            array_start_timestamp <= fragment_timestamp_end) ||
-           (array_end_timestamp < fragment_timestamp_end &&
-            array_end_timestamp >= fragment_timestamp_start);
+    // This method assumes overlapping fragment and array times so checking that
+    // we don't have full overlap is sufficient for detecting partial overlap.
+    auto full_fragment_overlap =
+        (fragment_timestamp_start >= array_start_timestamp) &&
+        (fragment_timestamp_end <= array_end_timestamp);
+    return !full_fragment_overlap;
   }
 
   /**
@@ -836,7 +868,8 @@ class FragmentMetadata {
     std::vector<uint64_t> tile_max_offsets_;
     std::vector<uint64_t> tile_sum_offsets_;
     std::vector<uint64_t> tile_null_count_offsets_;
-    uint64_t tile_min_max_sum_null_count_offset_;
+    uint64_t fragment_min_max_sum_null_count_offset_;
+    uint64_t processed_conditions_offsets_;
   };
 
   /** Keeps track of which metadata is loaded. */
@@ -852,6 +885,7 @@ class FragmentMetadata {
     std::vector<bool> tile_sum_;
     std::vector<bool> tile_null_count_;
     bool fragment_min_max_sum_null_count_ = false;
+    bool processed_conditions_ = false;
   };
 
   /* ********************************* */
@@ -920,6 +954,9 @@ class FragmentMetadata {
   /** True if the fragment has timestamps, and false otherwise. */
   bool has_timestamps_;
 
+  /** True if the fragment has delete metadata, and false otherwise. */
+  bool has_delete_meta_;
+
   /** Number of sparse tiles. */
   uint64_t sparse_tile_num_;
 
@@ -1038,6 +1075,9 @@ class FragmentMetadata {
   /** The uri of the array the metadata belongs to. */
   URI array_uri_;
 
+  /** Set of already processed delete/update conditions for this fragment. */
+  std::unordered_set<std::string> processed_conditions_;
+
   /* ********************************* */
   /*           PRIVATE METHODS         */
   /* ********************************* */
@@ -1251,6 +1291,14 @@ class FragmentMetadata {
    */
   Status load_has_timestamps(ConstBuffer* buff);
 
+  /**
+   * Loads the `has_delete_meta_` field from the buffer.
+   *
+   * @param buff Metadata buffer.
+   * @return Status
+   */
+  Status load_has_delete_meta(ConstBuffer* buff);
+
   /**
    * Loads the MBRs from the fragment metadata buffer.
    *
@@ -1345,6 +1393,11 @@ class FragmentMetadata {
    */
   Status load_fragment_min_max_sum_null_count(ConstBuffer* buff);
 
+  /**
+   * Loads the processed conditions for the fragment.
+   */
+  Status load_processed_conditions(ConstBuffer* buff);
+
   /** Loads the format version from the buffer. */
   Status load_version(ConstBuffer* buff);
 
@@ -1407,6 +1460,11 @@ class FragmentMetadata {
    */
   Status write_has_timestamps(Buffer* buff) const;
 
+  /**
+   * Writes the `has_delete_meta_` field to the fragment metadata buffer.
+   */
+  Status write_has_delete_meta(Buffer* buff) const;
+
   /**
    * Writes the R-tree to storage.
    *
@@ -1571,6 +1629,16 @@ class FragmentMetadata {
   Status store_fragment_min_max_sum_null_count(
       uint64_t num, const EncryptionKey& encryption_key, uint64_t* nbytes);
 
+  /**
+   * Writes the processed conditions to storage.
+   *
+   * @param encryption_key The encryption key.
+   * @param nbytes The total number of bytes written.
+   * @return Status
+   */
+  Status store_processed_conditions(
+      const EncryptionKey& encryption_key, uint64_t* nbytes);
+
   /**
    * Compute the fragment min, max and sum values.
    *
@@ -1608,8 +1676,8 @@ class FragmentMetadata {
    * Reads the contents of a generic tile starting at the input offset,
    * and stores them into buffer ``buff``.
    */
-  Status read_generic_tile_from_file(
-      const EncryptionKey& encryption_key, uint64_t offset, Buffer* buff) const;
+  tuple<Status, optional<Buffer>> read_generic_tile_from_file(
+      const EncryptionKey& encryption_key, uint64_t offset) const;
 
   /**
    * Reads the fragment metadata file footer (which contains the generic tile
