@@ -84,7 +84,8 @@ Query::Query(
     , offsets_buffer_name_("")
     , disable_checks_consolidation_(false)
     , consolidation_with_timestamps_(false)
-    , fragment_uri_(fragment_uri) {
+    , fragment_uri_(fragment_uri)
+    , force_legacy_reader_(false) {
   assert(array->is_open());
   auto st = array->get_query_type(&type_);
   assert(st.ok());
@@ -1087,6 +1088,17 @@ Status Query::process() {
   return Status::Ok();
 }
 
+Status Query::reset_strategy_with_layout(
+    Layout layout, bool force_legacy_reader) {
+  force_legacy_reader_ = force_legacy_reader;
+  strategy_ = nullptr;
+  layout_ = layout;
+  subarray_.set_layout(layout);
+  RETURN_NOT_OK(create_strategy());
+
+  return Status::Ok();
+}
+
 Status Query::create_strategy() {
   if (type_ == QueryType::WRITE) {
     if (layout_ == Layout::COL_MAJOR || layout_ == Layout::ROW_MAJOR) {
@@ -1260,10 +1272,6 @@ IQueryStrategy* Query::strategy() {
   return strategy_.get();
 }
 
-void Query::clear_strategy() {
-  strategy_ = nullptr;
-}
-
 Status Query::disable_checks_consolidation() {
   if (status_ != QueryStatus::UNINITIALIZED) {
     return logger_->status(Status_QueryError(
@@ -2224,6 +2232,12 @@ bool Query::use_refactored_dense_reader(
     const ArraySchema& array_schema, bool all_dense) {
   bool use_refactored_reader = false;
   bool found = false;
+
+  // If the query comes from a client using the legacy reader.
+  if (force_legacy_reader_) {
+    return false;
+  }
+
   // First check for legacy option
   config_.get<bool>(
       "sm.use_refactored_readers", &use_refactored_reader, &found);
@@ -2246,6 +2260,12 @@ bool Query::use_refactored_sparse_global_order_reader(
     Layout layout, const ArraySchema& array_schema) {
   bool use_refactored_reader = false;
   bool found = false;
+
+  // If the query comes from a client using the legacy reader.
+  if (force_legacy_reader_) {
+    return false;
+  }
+
   // First check for legacy option
   config_.get<bool>(
       "sm.use_refactored_readers", &use_refactored_reader, &found);
@@ -2271,6 +2291,11 @@ bool Query::use_refactored_sparse_unordered_with_dups_reader(
   bool use_refactored_reader = false;
   bool found = false;
 
+  // If the query comes from a client using the legacy reader.
+  if (force_legacy_reader_) {
+    return false;
+  }
+
   // First check for legacy option
   config_.get<bool>(
       "sm.use_refactored_readers", &use_refactored_reader, &found);
