@@ -557,8 +557,15 @@ class Query {
   /** Gets the strategy of the query. */
   IQueryStrategy* strategy();
 
-  /** Remove the current strategy. Used by serialization. */
-  void clear_strategy();
+  /**
+   * Switch the strategy depending on layout. Used by serialization.
+   *
+   * @param layout New layout
+   * @param force_legacy_reader Force use of the legacy reader if the client
+   *    requested it.
+   * @return Status
+   */
+  Status reset_strategy_with_layout(Layout layout, bool force_legacy_reader);
 
   /**
    * Disables checking the global order and coordinate duplicates. Applicable
@@ -1009,6 +1016,14 @@ class Query {
   /* Scratch space used for REST requests. */
   shared_ptr<Buffer> rest_scratch_;
 
+  /**
+   * Flag to force legacy reader when strategy gets created. This is used by
+   * the serialization codebase if a query comes from an older version of the
+   * library that doesn't have the refactored readers, we need to run it with
+   * the legacy reader.
+   */
+  bool force_legacy_reader_;
+
   /* ********************************* */
   /*           PRIVATE METHODS         */
   /* ********************************* */
