@@ -44,6 +44,7 @@
 #include <memory>
 #include <mutex>
 #include <numeric>
+#include <type_traits>
 
 using namespace tiledb::common;
 
@@ -53,6 +54,10 @@ namespace sm {
 QueryCondition::QueryCondition() {
 }
 
+QueryCondition::QueryCondition(tdb_unique_ptr<tiledb::sm::ASTNode>&& tree)
+    : tree_(std::move(tree)) {
+}
+
 QueryCondition::QueryCondition(const QueryCondition& rhs)
     : tree_(rhs.tree_ == nullptr ? nullptr : rhs.tree_->clone()) {
 }
@@ -432,6 +437,7 @@ void QueryCondition::apply_ast_node(
               cell_size,
               condition_value_content,
               condition_value_size);
+
           result_cell_bitmap[starting_index + c] = combination_op(
               result_cell_bitmap[starting_index + c], (uint8_t)cmp);
           ++c;
@@ -536,17 +542,25 @@ void QueryCondition::apply_ast_node(
     const std::vector<ResultCellSlab>& result_cell_slabs,
     CombinationOp combination_op,
     std::vector<uint8_t>& result_cell_bitmap) const {
-  const auto attribute = array_schema.attribute(node->get_field_name());
-  if (!attribute) {
-    throw std::runtime_error(
-        "QueryCondition::apply_ast_node: Unknown attribute " +
-        node->get_field_name());
+  std::string node_field_name = node->get_field_name();
+
+  const auto nullable = array_schema.is_nullable(node_field_name);
+  const auto var_size = array_schema.var_size(node_field_name);
+  const auto type = array_schema.type(node_field_name);
+
+  ByteVecValue fill_value;
+  if (node_field_name != constants::timestamps &&
+      node_field_name != constants::delete_timestamps) {
+    if (!array_schema.is_dim(node_field_name)) {
+      const auto attribute = array_schema.attribute(node_field_name);
+      if (!attribute) {
+        throw std::runtime_error("Unknown attribute " + node_field_name);
+      }
+      fill_value = attribute->fill_value();
+    }
   }
 
-  const ByteVecValue fill_value = attribute->fill_value();
-  const bool var_size = attribute->var_size();
-  const bool nullable = attribute->nullable();
-  switch (attribute->type()) {
+  switch (type) {
     case Datatype::INT8: {
       apply_ast_node<int8_t, CombinationOp>(
           node,
@@ -891,6 +905,7 @@ void QueryCondition::apply_ast_node_dense(
     const uint64_t src_cell,
     const uint64_t stride,
     const bool var_size,
+    const bool nullable,
     CombinationOp combination_op,
     span<uint8_t> result_buffer) const {
   const std::string& field_name = node->get_field_name();
@@ -900,6 +915,11 @@ void QueryCondition::apply_ast_node_dense(
 
   // Get the nullable buffer.
   const auto tile_tuple = result_tile->tile_tuple(field_name);
+  uint8_t* buffer_validity = nullptr;
+  if (nullable) {
+    const auto& tile_validity = std::get<2>(*tile_tuple);
+    buffer_validity = static_cast<uint8_t*>(tile_validity.data()) + src_cell;
+  }
 
   if (var_size) {
     // Get var data buffer and tile offsets buffer.
@@ -933,7 +953,11 @@ void QueryCondition::apply_ast_node_dense(
           cell_value, cell_size, condition_value_content, condition_value_size);
 
       // Set the value.
-      result_buffer[c] = combination_op(result_buffer[c], (uint8_t)cmp);
+      bool buffer_validity_val = buffer_validity == nullptr ?
+                                     true :
+                                     buffer_validity[start + c * stride] != 0;
+      result_buffer[c] =
+          combination_op(result_buffer[c], (uint8_t)cmp && buffer_validity_val);
     }
   } else {
     // Get the fixed size data buffers.
@@ -954,7 +978,11 @@ void QueryCondition::apply_ast_node_dense(
           cell_value, cell_size, condition_value_content, condition_value_size);
 
       // Set the value.
-      result_buffer[c] = combination_op(result_buffer[c], (uint8_t)cmp);
+      bool buffer_validity_val = buffer_validity == nullptr ?
+                                     true :
+                                     buffer_validity[start + c * stride] != 0;
+      result_buffer[c] =
+          combination_op(result_buffer[c], (uint8_t)cmp && buffer_validity_val);
     }
   }
 }
@@ -967,6 +995,7 @@ void QueryCondition::apply_ast_node_dense(
     const uint64_t src_cell,
     const uint64_t stride,
     const bool var_size,
+    const bool nullable,
     CombinationOp combination_op,
     span<uint8_t> result_buffer) const {
   switch (node->get_op()) {
@@ -978,6 +1007,7 @@ void QueryCondition::apply_ast_node_dense(
           src_cell,
           stride,
           var_size,
+          nullable,
           combination_op,
           result_buffer);
       break;
@@ -989,6 +1019,7 @@ void QueryCondition::apply_ast_node_dense(
           src_cell,
           stride,
           var_size,
+          nullable,
           combination_op,
           result_buffer);
       break;
@@ -1000,6 +1031,7 @@ void QueryCondition::apply_ast_node_dense(
           src_cell,
           stride,
           var_size,
+          nullable,
           combination_op,
           result_buffer);
       break;
@@ -1011,6 +1043,7 @@ void QueryCondition::apply_ast_node_dense(
           src_cell,
           stride,
           var_size,
+          nullable,
           combination_op,
           result_buffer);
       break;
@@ -1022,6 +1055,7 @@ void QueryCondition::apply_ast_node_dense(
           src_cell,
           stride,
           var_size,
+          nullable,
           combination_op,
           result_buffer);
       break;
@@ -1033,6 +1067,7 @@ void QueryCondition::apply_ast_node_dense(
           src_cell,
           stride,
           var_size,
+          nullable,
           combination_op,
           result_buffer);
       break;
@@ -1061,165 +1096,186 @@ void QueryCondition::apply_ast_node_dense(
   const bool nullable = attribute->nullable();
 
   // Process the validity buffer now.
-  if (nullable) {
+  if (nullable && node->get_condition_value_view().content() == nullptr) {
     const auto tile_tuple = result_tile->tile_tuple(node->get_field_name());
     const auto& tile_validity = std::get<2>(*tile_tuple);
     const auto buffer_validity =
         static_cast<uint8_t*>(tile_validity.data()) + src_cell;
 
     // Null values can only be specified for equality operators.
-    if (node->get_condition_value_view().content() == nullptr) {
-      if (node->get_op() == QueryConditionOp::NE) {
-        for (uint64_t c = 0; c < result_buffer.size(); ++c) {
-          result_buffer[c] *= buffer_validity[start + c * stride] != 0;
-        }
-      } else {
-        for (uint64_t c = 0; c < result_buffer.size(); ++c) {
-          result_buffer[c] *= buffer_validity[start + c * stride] == 0;
-        }
-      }
-      return;
-    } else {
-      // Turn off bitmap values for null cells.
+    if (node->get_op() == QueryConditionOp::NE) {
       for (uint64_t c = 0; c < result_buffer.size(); ++c) {
-        result_buffer[c] *= buffer_validity[start + c * stride] != 0;
+        result_buffer[c] = combination_op(
+            buffer_validity[start + c * stride] != 0, result_buffer[c]);
+      }
+    } else {
+      for (uint64_t c = 0; c < result_buffer.size(); ++c) {
+        result_buffer[c] = combination_op(
+            buffer_validity[start + c * stride] == 0, result_buffer[c]);
       }
     }
+    return;
   }
 
   switch (attribute->type()) {
-    case Datatype::INT8:
-      return apply_ast_node_dense<int8_t, CombinationOp>(
+    case Datatype::INT8: {
+      apply_ast_node_dense<int8_t, CombinationOp>(
           node,
           result_tile,
           start,
           src_cell,
           stride,
           var_size,
+          nullable,
           combination_op,
           result_buffer);
+    } break;
     case Datatype::BOOL:
-    case Datatype::UINT8:
-      return apply_ast_node_dense<uint8_t, CombinationOp>(
+    case Datatype::UINT8: {
+      apply_ast_node_dense<uint8_t, CombinationOp>(
           node,
           result_tile,
           start,
           src_cell,
           stride,
           var_size,
+          nullable,
           combination_op,
           result_buffer);
-    case Datatype::INT16:
-      return apply_ast_node_dense<int16_t, CombinationOp>(
+    } break;
+    case Datatype::INT16: {
+      apply_ast_node_dense<int16_t, CombinationOp>(
           node,
           result_tile,
           start,
           src_cell,
           stride,
           var_size,
+          nullable,
           combination_op,
           result_buffer);
-    case Datatype::UINT16:
-      return apply_ast_node_dense<uint16_t, CombinationOp>(
+    } break;
+    case Datatype::UINT16: {
+      apply_ast_node_dense<uint16_t, CombinationOp>(
           node,
           result_tile,
           start,
           src_cell,
           stride,
           var_size,
+          nullable,
           combination_op,
           result_buffer);
-    case Datatype::INT32:
-      return apply_ast_node_dense<int32_t, CombinationOp>(
+    } break;
+    case Datatype::INT32: {
+      apply_ast_node_dense<int32_t, CombinationOp>(
           node,
           result_tile,
           start,
           src_cell,
           stride,
           var_size,
+          nullable,
           combination_op,
           result_buffer);
-    case Datatype::UINT32:
-      return apply_ast_node_dense<uint32_t, CombinationOp>(
+    } break;
+    case Datatype::UINT32: {
+      apply_ast_node_dense<uint32_t, CombinationOp>(
           node,
           result_tile,
           start,
           src_cell,
           stride,
           var_size,
+          nullable,
           combination_op,
           result_buffer);
-    case Datatype::INT64:
-      return apply_ast_node_dense<int64_t, CombinationOp>(
+    } break;
+    case Datatype::INT64: {
+      apply_ast_node_dense<int64_t, CombinationOp>(
           node,
           result_tile,
           start,
           src_cell,
           stride,
           var_size,
+          nullable,
           combination_op,
           result_buffer);
-    case Datatype::UINT64:
-      return apply_ast_node_dense<uint64_t, CombinationOp>(
+    } break;
+    case Datatype::UINT64: {
+      apply_ast_node_dense<uint64_t, CombinationOp>(
           node,
           result_tile,
           start,
           src_cell,
           stride,
           var_size,
+          nullable,
           combination_op,
           result_buffer);
-    case Datatype::FLOAT32:
-      return apply_ast_node_dense<float, CombinationOp>(
+    } break;
+    case Datatype::FLOAT32: {
+      apply_ast_node_dense<float, CombinationOp>(
           node,
           result_tile,
           start,
           src_cell,
           stride,
           var_size,
+          nullable,
           combination_op,
           result_buffer);
-    case Datatype::FLOAT64:
-      return apply_ast_node_dense<double, CombinationOp>(
+    } break;
+    case Datatype::FLOAT64: {
+      apply_ast_node_dense<double, CombinationOp>(
           node,
           result_tile,
           start,
           src_cell,
           stride,
           var_size,
+          nullable,
           combination_op,
           result_buffer);
-    case Datatype::STRING_ASCII:
-      return apply_ast_node_dense<char*, CombinationOp>(
+    } break;
+    case Datatype::STRING_ASCII: {
+      apply_ast_node_dense<char*, CombinationOp>(
           node,
           result_tile,
           start,
           src_cell,
           stride,
           var_size,
+          nullable,
           combination_op,
           result_buffer);
-    case Datatype::CHAR:
+    } break;
+    case Datatype::CHAR: {
       if (var_size) {
-        return apply_ast_node_dense<char*, CombinationOp>(
+        apply_ast_node_dense<char*, CombinationOp>(
             node,
             result_tile,
             start,
             src_cell,
             stride,
             var_size,
+            nullable,
+            combination_op,
+            result_buffer);
+      } else {
+        apply_ast_node_dense<char, CombinationOp>(
+            node,
+            result_tile,
+            start,
+            src_cell,
+            stride,
+            var_size,
+            nullable,
             combination_op,
             result_buffer);
       }
-      return apply_ast_node_dense<char, CombinationOp>(
-          node,
-          result_tile,
-          start,
-          src_cell,
-          stride,
-          var_size,
-          combination_op,
-          result_buffer);
+    } break;
     case Datatype::DATETIME_YEAR:
     case Datatype::DATETIME_MONTH:
     case Datatype::DATETIME_WEEK:
@@ -1232,16 +1288,18 @@ void QueryCondition::apply_ast_node_dense(
     case Datatype::DATETIME_NS:
     case Datatype::DATETIME_PS:
     case Datatype::DATETIME_FS:
-    case Datatype::DATETIME_AS:
-      return apply_ast_node_dense<int64_t, CombinationOp>(
+    case Datatype::DATETIME_AS: {
+      apply_ast_node_dense<int64_t, CombinationOp>(
           node,
           result_tile,
           start,
           src_cell,
           stride,
           var_size,
+          nullable,
           combination_op,
           result_buffer);
+    } break;
     case Datatype::ANY:
     case Datatype::BLOB:
     case Datatype::STRING_UTF8:
@@ -1553,7 +1611,8 @@ template <
     typename T,
     QueryConditionOp Op,
     typename BitmapType,
-    typename CombinationOp>
+    typename CombinationOp,
+    typename nullable>
 void QueryCondition::apply_ast_node_sparse(
     const tdb_unique_ptr<ASTNode>& node,
     ResultTile& result_tile,
@@ -1564,6 +1623,14 @@ void QueryCondition::apply_ast_node_sparse(
   const void* condition_value_content =
       node->get_condition_value_view().content();
   const size_t condition_value_size = node->get_condition_value_view().size();
+  uint8_t* buffer_validity = nullptr;
+
+  // Check if the combination op = OR and the attribute is nullable.
+  if constexpr (
+      std::is_same_v<CombinationOp, QCMax<BitmapType>> && nullable::value) {
+    const auto& tile_validity = std::get<2>(*tile_tuple);
+    buffer_validity = static_cast<uint8_t*>(tile_validity.data());
+  }
 
   if (var_size) {
     // Get var data buffer and tile offsets buffer.
@@ -1594,8 +1661,15 @@ void QueryCondition::apply_ast_node_sparse(
       const bool cmp = BinaryCmp<T, Op>::cmp(
           cell_value, cell_size, condition_value_content, condition_value_size);
 
-      // Set the value.
-      result_bitmap[c] = combination_op(result_bitmap[c], cmp);
+      // Set the value, casing on whether the combination op = OR and the
+      // attribute is nullable.
+      if constexpr (
+          std::is_same_v<CombinationOp, QCMax<BitmapType>> && nullable::value) {
+        result_bitmap[c] =
+            combination_op(result_bitmap[c], cmp && (buffer_validity[c] != 0));
+      } else {
+        result_bitmap[c] = combination_op(result_bitmap[c], cmp);
+      }
     }
   } else {
     // Get the fixed size data buffers.
@@ -1614,13 +1688,24 @@ void QueryCondition::apply_ast_node_sparse(
       const bool cmp = BinaryCmp<T, Op>::cmp(
           cell_value, cell_size, condition_value_content, condition_value_size);
 
-      // Set the value.
-      result_bitmap[c] = combination_op(result_bitmap[c], cmp);
+      // Set the value, casing on whether the combination op = OR and the
+      // attribute is nullable.
+      if constexpr (
+          std::is_same_v<CombinationOp, QCMax<BitmapType>> && nullable::value) {
+        result_bitmap[c] =
+            combination_op(result_bitmap[c], cmp && (buffer_validity[c] != 0));
+      } else {
+        result_bitmap[c] = combination_op(result_bitmap[c], cmp);
+      }
     }
   }
 }
 
-template <typename T, typename BitmapType, typename CombinationOp>
+template <
+    typename T,
+    typename BitmapType,
+    typename CombinationOp,
+    typename nullable>
 void QueryCondition::apply_ast_node_sparse(
     const tdb_unique_ptr<ASTNode>& node,
     ResultTile& result_tile,
@@ -1629,28 +1714,52 @@ void QueryCondition::apply_ast_node_sparse(
     std::vector<BitmapType>& result_bitmap) const {
   switch (node->get_op()) {
     case QueryConditionOp::LT:
-      apply_ast_node_sparse<T, QueryConditionOp::LT, BitmapType>(
-          node, result_tile, var_size, combination_op, result_bitmap);
+      apply_ast_node_sparse<
+          T,
+          QueryConditionOp::LT,
+          BitmapType,
+          CombinationOp,
+          nullable>(node, result_tile, var_size, combination_op, result_bitmap);
       break;
     case QueryConditionOp::LE:
-      apply_ast_node_sparse<T, QueryConditionOp::LE, BitmapType>(
-          node, result_tile, var_size, combination_op, result_bitmap);
+      apply_ast_node_sparse<
+          T,
+          QueryConditionOp::LE,
+          BitmapType,
+          CombinationOp,
+          nullable>(node, result_tile, var_size, combination_op, result_bitmap);
       break;
     case QueryConditionOp::GT:
-      apply_ast_node_sparse<T, QueryConditionOp::GT, BitmapType>(
-          node, result_tile, var_size, combination_op, result_bitmap);
+      apply_ast_node_sparse<
+          T,
+          QueryConditionOp::GT,
+          BitmapType,
+          CombinationOp,
+          nullable>(node, result_tile, var_size, combination_op, result_bitmap);
       break;
     case QueryConditionOp::GE:
-      apply_ast_node_sparse<T, QueryConditionOp::GE, BitmapType>(
-          node, result_tile, var_size, combination_op, result_bitmap);
+      apply_ast_node_sparse<
+          T,
+          QueryConditionOp::GE,
+          BitmapType,
+          CombinationOp,
+          nullable>(node, result_tile, var_size, combination_op, result_bitmap);
       break;
     case QueryConditionOp::EQ:
-      apply_ast_node_sparse<T, QueryConditionOp::EQ, BitmapType>(
-          node, result_tile, var_size, combination_op, result_bitmap);
+      apply_ast_node_sparse<
+          T,
+          QueryConditionOp::EQ,
+          BitmapType,
+          CombinationOp,
+          nullable>(node, result_tile, var_size, combination_op, result_bitmap);
       break;
     case QueryConditionOp::NE:
-      apply_ast_node_sparse<T, QueryConditionOp::NE, BitmapType>(
-          node, result_tile, var_size, combination_op, result_bitmap);
+      apply_ast_node_sparse<
+          T,
+          QueryConditionOp::NE,
+          BitmapType,
+          CombinationOp,
+          nullable>(node, result_tile, var_size, combination_op, result_bitmap);
       break;
     default:
       throw std::runtime_error(
@@ -1658,6 +1767,23 @@ void QueryCondition::apply_ast_node_sparse(
   }
 }
 
+template <typename T, typename BitmapType, typename CombinationOp>
+void QueryCondition::apply_ast_node_sparse(
+    const tdb_unique_ptr<ASTNode>& node,
+    ResultTile& result_tile,
+    const bool var_size,
+    const bool nullable,
+    CombinationOp combination_op,
+    std::vector<BitmapType>& result_bitmap) const {
+  if (nullable) {
+    apply_ast_node_sparse<T, BitmapType, CombinationOp, std::true_type>(
+        node, result_tile, var_size, combination_op, result_bitmap);
+  } else {
+    apply_ast_node_sparse<T, BitmapType, CombinationOp, std::false_type>(
+        node, result_tile, var_size, combination_op, result_bitmap);
+  }
+}
+
 template <typename BitmapType, typename CombinationOp>
 void QueryCondition::apply_ast_node_sparse(
     const tdb_unique_ptr<ASTNode>& node,
@@ -1665,43 +1791,36 @@ void QueryCondition::apply_ast_node_sparse(
     ResultTile& result_tile,
     CombinationOp combination_op,
     std::vector<BitmapType>& result_bitmap) const {
-  bool var_size = false, nullable = false;
-
-  // Initialize to timestamps type.
-  Datatype type = Datatype::UINT64;
   std::string node_field_name = node->get_field_name();
-  if (node_field_name != constants::timestamps) {
-    const auto attribute = array_schema.attribute(node_field_name);
-    if (!attribute) {
-      throw std::runtime_error("Unknown attribute " + node_field_name);
-    }
-
-    var_size = attribute->var_size();
-    nullable = attribute->nullable();
-    type = attribute->type();
-  }
+  const auto nullable = array_schema.is_nullable(node_field_name);
+  const auto var_size = array_schema.var_size(node_field_name);
+  const auto type = array_schema.type(node_field_name);
 
   // Process the validity buffer now.
   if (nullable) {
-    const auto tile_tuple = result_tile.tile_tuple(node->get_field_name());
+    const auto tile_tuple = result_tile.tile_tuple(node_field_name);
     const auto& tile_validity = std::get<2>(*tile_tuple);
     const auto buffer_validity = static_cast<uint8_t*>(tile_validity.data());
-
-    // Null values can only be specified for equality operators.
     const auto cell_num = result_tile.cell_num();
+
     if (node->get_condition_value_view().content() == nullptr) {
+      // Null values can only be specified for equality operators.
       if (node->get_op() == QueryConditionOp::NE) {
         for (uint64_t c = 0; c < cell_num; c++) {
-          result_bitmap[c] *= buffer_validity[c] != 0;
+          result_bitmap[c] =
+              combination_op(buffer_validity[c] != 0, result_bitmap[c]);
         }
       } else {
         for (uint64_t c = 0; c < cell_num; c++) {
-          result_bitmap[c] *= buffer_validity[c] == 0;
+          result_bitmap[c] =
+              combination_op(buffer_validity[c] == 0, result_bitmap[c]);
         }
       }
       return;
-    } else {
-      // Turn off bitmap values for null cells.
+    } else if constexpr (std::is_same_v<
+                             CombinationOp,
+                             std::multiplies<BitmapType>>) {
+      // When the combination op is AND, turn off bitmap values for null cells.
       for (uint64_t c = 0; c < cell_num; c++) {
         result_bitmap[c] *= buffer_validity[c] != 0;
       }
@@ -1709,46 +1828,69 @@ void QueryCondition::apply_ast_node_sparse(
   }
 
   switch (type) {
-    case Datatype::INT8:
-      return apply_ast_node_sparse<int8_t, BitmapType>(
-          node, result_tile, var_size, combination_op, result_bitmap);
-    case Datatype::UINT8:
-      return apply_ast_node_sparse<uint8_t, BitmapType>(
-          node, result_tile, var_size, combination_op, result_bitmap);
-    case Datatype::INT16:
-      return apply_ast_node_sparse<int16_t, BitmapType>(
-          node, result_tile, var_size, combination_op, result_bitmap);
-    case Datatype::UINT16:
-      return apply_ast_node_sparse<uint16_t, BitmapType>(
-          node, result_tile, var_size, combination_op, result_bitmap);
-    case Datatype::INT32:
-      return apply_ast_node_sparse<int32_t, BitmapType>(
-          node, result_tile, var_size, combination_op, result_bitmap);
-    case Datatype::UINT32:
-      return apply_ast_node_sparse<uint32_t, BitmapType>(
-          node, result_tile, var_size, combination_op, result_bitmap);
-    case Datatype::INT64:
-      return apply_ast_node_sparse<int64_t, BitmapType>(
-          node, result_tile, var_size, combination_op, result_bitmap);
-    case Datatype::UINT64:
-      return apply_ast_node_sparse<uint64_t, BitmapType>(
-          node, result_tile, var_size, combination_op, result_bitmap);
-    case Datatype::FLOAT32:
-      return apply_ast_node_sparse<float, BitmapType>(
-          node, result_tile, var_size, combination_op, result_bitmap);
-    case Datatype::FLOAT64:
-      return apply_ast_node_sparse<double, BitmapType>(
-          node, result_tile, var_size, combination_op, result_bitmap);
-    case Datatype::STRING_ASCII:
-      return apply_ast_node_sparse<char*, BitmapType>(
-          node, result_tile, var_size, combination_op, result_bitmap);
-    case Datatype::CHAR:
+    case Datatype::INT8: {
+      apply_ast_node_sparse<int8_t, BitmapType, CombinationOp>(
+          node, result_tile, var_size, nullable, combination_op, result_bitmap);
+    } break;
+    case Datatype::UINT8: {
+      apply_ast_node_sparse<uint8_t, BitmapType, CombinationOp>(
+          node, result_tile, var_size, nullable, combination_op, result_bitmap);
+    } break;
+    case Datatype::INT16: {
+      apply_ast_node_sparse<int16_t, BitmapType, CombinationOp>(
+          node, result_tile, var_size, nullable, combination_op, result_bitmap);
+    } break;
+    case Datatype::UINT16: {
+      apply_ast_node_sparse<uint16_t, BitmapType, CombinationOp>(
+          node, result_tile, var_size, nullable, combination_op, result_bitmap);
+    } break;
+    case Datatype::INT32: {
+      apply_ast_node_sparse<int32_t, BitmapType, CombinationOp>(
+          node, result_tile, var_size, nullable, combination_op, result_bitmap);
+    } break;
+    case Datatype::UINT32: {
+      apply_ast_node_sparse<uint32_t, BitmapType, CombinationOp>(
+          node, result_tile, var_size, nullable, combination_op, result_bitmap);
+    } break;
+    case Datatype::INT64: {
+      apply_ast_node_sparse<int64_t, BitmapType, CombinationOp>(
+          node, result_tile, var_size, nullable, combination_op, result_bitmap);
+    } break;
+    case Datatype::UINT64: {
+      apply_ast_node_sparse<uint64_t, BitmapType, CombinationOp>(
+          node, result_tile, var_size, nullable, combination_op, result_bitmap);
+    } break;
+    case Datatype::FLOAT32: {
+      apply_ast_node_sparse<float, BitmapType, CombinationOp>(
+          node, result_tile, var_size, nullable, combination_op, result_bitmap);
+    } break;
+    case Datatype::FLOAT64: {
+      apply_ast_node_sparse<double, BitmapType, CombinationOp>(
+          node, result_tile, var_size, nullable, combination_op, result_bitmap);
+    } break;
+    case Datatype::STRING_ASCII: {
+      apply_ast_node_sparse<char*, BitmapType, CombinationOp>(
+          node, result_tile, var_size, nullable, combination_op, result_bitmap);
+    } break;
+    case Datatype::CHAR: {
       if (var_size) {
-        return apply_ast_node_sparse<char*, BitmapType>(
-            node, result_tile, var_size, combination_op, result_bitmap);
+        apply_ast_node_sparse<char*, BitmapType, CombinationOp>(
+            node,
+            result_tile,
+            var_size,
+            nullable,
+            combination_op,
+            result_bitmap);
+      } else {
+        apply_ast_node_sparse<char, BitmapType, CombinationOp>(
+            node,
+            result_tile,
+            var_size,
+            nullable,
+            combination_op,
+            result_bitmap);
       }
-      return apply_ast_node_sparse<char, BitmapType>(
-          node, result_tile, var_size, combination_op, result_bitmap);
+    } break;
     case Datatype::DATETIME_YEAR:
     case Datatype::DATETIME_MONTH:
     case Datatype::DATETIME_WEEK:
@@ -1761,9 +1903,10 @@ void QueryCondition::apply_ast_node_sparse(
     case Datatype::DATETIME_NS:
     case Datatype::DATETIME_PS:
     case Datatype::DATETIME_FS:
-    case Datatype::DATETIME_AS:
-      return apply_ast_node_sparse<int64_t, BitmapType>(
-          node, result_tile, var_size, combination_op, result_bitmap);
+    case Datatype::DATETIME_AS: {
+      apply_ast_node_sparse<int64_t, BitmapType, CombinationOp>(
+          node, result_tile, var_size, nullable, combination_op, result_bitmap);
+    } break;
     case Datatype::ANY:
     case Datatype::BLOB:
     case Datatype::STRING_UTF8:
@@ -1821,7 +1964,8 @@ void QueryCondition::apply_tree_sparse(
                 result_bitmap);
           }
 
-          // Handle the cl'(q, a) case
+          // Handle the cl'(q, a) case.
+          // This cases on whether the combination op = OR.
         } else if constexpr (std::is_same_v<CombinationOp, QCMax<BitmapType>>) {
           std::vector<BitmapType> combination_op_bitmap(result_bitmap_size, 1);
 
@@ -1892,6 +2036,10 @@ Status QueryCondition::apply_sparse(
   return Status::Ok();
 }
 
+QueryCondition QueryCondition::negated_condition() {
+  return QueryCondition(tree_->get_negated_tree());
+}
+
 const tdb_unique_ptr<ASTNode>& QueryCondition::ast() const {
   return tree_;
 }
