@@ -223,6 +223,11 @@ Status SparseGlobalOrderReader<BitmapType>::dowork() {
       // Run deduplication for tiles with timestamps, if required.
       RETURN_NOT_OK(dedup_tiles_with_timestamps(tmp_result_tiles));
 
+      // Compute hilbert values.
+      if (array_schema_.cell_order() == Layout::HILBERT) {
+        RETURN_NOT_OK(compute_hilbert_values(tmp_result_tiles));
+      }
+
       // Clear result tiles that are not necessary anymore.
       std::mutex ignored_tiles_mutex;
       auto status = parallel_for(
@@ -243,11 +248,6 @@ Status SparseGlobalOrderReader<BitmapType>::dowork() {
             return Status::Ok();
           });
       RETURN_NOT_OK_ELSE(status, logger_->status(status));
-
-      // Compute hilbert values.
-      if (array_schema_.cell_order() == Layout::HILBERT) {
-        RETURN_NOT_OK(compute_hilbert_values(tmp_result_tiles));
-      }
     }
 
     // For fragments with timestamps, check first and last cell of every tiles
@@ -1058,7 +1058,7 @@ Status SparseGlobalOrderReader<BitmapType>::copy_offsets_tiles(
     const std::vector<ResultCellSlab>& result_cell_slabs,
     const std::vector<uint64_t>& cell_offsets,
     QueryBuffer& query_buffer,
-    std::vector<void*>& var_data) {
+    std::vector<const void*>& var_data) {
   auto timer_se = stats_->start_timer("copy_offsets_tiles");
 
   // Process all tiles/cells in parallel.
@@ -1075,14 +1075,29 @@ Status SparseGlobalOrderReader<BitmapType>::copy_offsets_tiles(
             result_cell_slabs[i].tile_);
 
         // Get source buffers.
-        const auto tile_tuple = rt->tile_tuple(name);
-        const auto t = &std::get<0>(*tile_tuple);
-        const auto t_var = &std::get<1>(*tile_tuple);
-        const auto src_buff = t->template data_as<uint64_t>();
-        const auto src_var_buff = t_var->template data_as<char>();
-        const auto t_val = &std::get<2>(*tile_tuple);
         const auto cell_num =
             fragment_metadata_[rt->frag_idx()]->cell_num(rt->tile_idx());
+        const auto tile_tuple = rt->tile_tuple(name);
+
+        // If the tile_tuple is null, this is a field added in schema
+        // evolution. Use the fill value.
+        const uint64_t* src_buff = nullptr;
+        const uint8_t* src_var_buff = nullptr;
+        bool use_fill_value = false;
+        OffType fill_value_size = 0;
+        uint64_t t_var_size = 0;
+        if (tile_tuple == nullptr) {
+          use_fill_value = true;
+          fill_value_size = static_cast<OffType>(
+              array_schema_.attribute(name)->fill_value().size());
+          src_var_buff = array_schema_.attribute(name)->fill_value().data();
+        } else {
+          const auto t = &std::get<0>(*tile_tuple);
+          const auto t_var = &std::get<1>(*tile_tuple);
+          t_var_size = t_var->size();
+          src_buff = t->template data_as<uint64_t>();
+          src_var_buff = t_var->template data_as<uint8_t>();
+        }
 
         // Compute parallelization parameters.
         auto&& [min_pos, max_pos, dest_cell_offset, skip_copy] =
@@ -1103,27 +1118,45 @@ Status SparseGlobalOrderReader<BitmapType>::copy_offsets_tiles(
         auto var_data_buffer = &var_data[dest_cell_offset - cell_offsets[0]];
 
         // Copy full tile. Last cell might be taken out for vectorization.
-        uint64_t end = (max_pos == cell_num) ? max_pos - 1 : max_pos;
-        for (uint64_t c = min_pos; c < end; c++) {
-          *buffer = (OffType)(src_buff[c + 1] - src_buff[c]) / offset_div;
-          buffer++;
-          *var_data_buffer = src_var_buff + src_buff[c];
-          var_data_buffer++;
+        uint64_t end =
+            (max_pos == cell_num && !use_fill_value) ? max_pos - 1 : max_pos;
+        if (!use_fill_value) {
+          for (uint64_t c = min_pos; c < end; c++) {
+            *buffer = (OffType)(src_buff[c + 1] - src_buff[c]) / offset_div;
+            buffer++;
+            *var_data_buffer = src_var_buff + src_buff[c];
+            var_data_buffer++;
+          }
+        } else {
+          for (uint64_t c = min_pos; c < end; c++) {
+            *buffer = fill_value_size / offset_div;
+            buffer++;
+            *var_data_buffer = src_var_buff;
+            var_data_buffer++;
+          }
         }
 
         // Copy last cell.
-        if (max_pos == cell_num) {
-          *buffer =
-              (OffType)(t_var->size() - src_buff[max_pos - 1]) / offset_div;
+        if (max_pos == cell_num && !use_fill_value) {
+          *buffer = (OffType)(t_var_size - src_buff[max_pos - 1]) / offset_div;
           *var_data_buffer = src_var_buff + src_buff[max_pos - 1];
         }
 
         // Copy nullable values.
         if (nullable) {
-          const auto src_val_buff = t_val->template data_as<uint8_t>();
-          for (uint64_t c = min_pos; c < max_pos; c++) {
-            *val_buffer = src_val_buff[c];
-            val_buffer++;
+          if (!use_fill_value) {
+            const auto& t_val = std::get<2>(*tile_tuple);
+            const auto src_val_buff = t_val.template data_as<uint8_t>();
+            for (uint64_t c = min_pos; c < max_pos; c++) {
+              *val_buffer = src_val_buff[c];
+              val_buffer++;
+            }
+          } else {
+            uint8_t v = array_schema_.attribute(name)->fill_value_validity();
+            for (uint64_t c = min_pos; c < max_pos; c++) {
+              *val_buffer = v;
+              val_buffer++;
+            }
           }
         }
 
@@ -1143,7 +1176,7 @@ Status SparseGlobalOrderReader<BitmapType>::copy_var_data_tiles(
     const std::vector<ResultCellSlab>& result_cell_slabs,
     const std::vector<uint64_t>& cell_offsets,
     QueryBuffer& query_buffer,
-    const std::vector<void*>& var_data) {
+    std::vector<const void*>& var_data) {
   auto timer_se = stats_->start_timer("copy_var_tiles");
 
   // For easy reference.
@@ -1241,9 +1274,18 @@ Status SparseGlobalOrderReader<BitmapType>::copy_fixed_data_tiles(
         const auto tile_tuple = stores_zipped_coords ?
                                     rt->tile_tuple(constants::coords) :
                                     rt->tile_tuple(name);
-        const auto t = &std::get<0>(*tile_tuple);
-        const auto src_buff = t->template data_as<uint8_t>();
-        const auto t_val = &std::get<2>(*tile_tuple);
+
+        // If the tile_tuple is null, this is a field added in schema
+        // evolution. Use the fill value.
+        const uint8_t* src_buff = nullptr;
+        bool use_fill_value = false;
+        if (tile_tuple == nullptr) {
+          use_fill_value = true;
+          src_buff = array_schema_.attribute(name)->fill_value().data();
+        } else {
+          const auto& t = std::get<0>(*tile_tuple);
+          src_buff = t.template data_as<uint8_t>();
+        }
 
         // Compute parallelization parameters.
         auto&& [min_pos, max_pos, dest_cell_offset, skip_copy] =
@@ -1264,11 +1306,19 @@ Status SparseGlobalOrderReader<BitmapType>::copy_fixed_data_tiles(
             query_buffer.validity_vector_.buffer() + dest_cell_offset;
 
         if (!stores_zipped_coords) {
-          // Copy tile.
-          memcpy(
-              buffer,
-              src_buff + min_pos * cell_size,
-              (max_pos - min_pos) * cell_size);
+          if (!use_fill_value) {
+            // Copy tile.
+            memcpy(
+                buffer,
+                src_buff + min_pos * cell_size,
+                (max_pos - min_pos) * cell_size);
+          } else {
+            // Copy tile using the fill value.
+            for (uint64_t i = 0; i < max_pos - min_pos; i++) {
+              memcpy(buffer, src_buff, cell_size);
+              buffer += cell_size;
+            }
+          }
         } else {  // Copy for zipped coords.
           const auto dim_num = rt->domain()->dim_num();
           for (uint64_t c = min_pos; c < max_pos; c++) {
@@ -1279,8 +1329,19 @@ Status SparseGlobalOrderReader<BitmapType>::copy_fixed_data_tiles(
         }
 
         if (nullable) {
-          const auto src_val_buff = t_val->template data_as<uint8_t>();
-          memcpy(val_buffer, src_val_buff + min_pos, max_pos - min_pos);
+          if (!use_fill_value) {
+            // Copy validity values from tile.
+            const auto& t_val = std::get<2>(*tile_tuple);
+            const auto src_val_buff = t_val.template data_as<uint8_t>();
+            memcpy(val_buffer, src_val_buff + min_pos, max_pos - min_pos);
+          } else {
+            // Copy the fill value for validity.
+            uint8_t v = array_schema_.attribute(name)->fill_value_validity();
+            for (uint64_t i = 0; i < max_pos - min_pos; i++) {
+              *val_buffer = v;
+              val_buffer++;
+            }
+          }
         }
 
         return Status::Ok();
@@ -1396,6 +1457,12 @@ SparseGlobalOrderReader<BitmapType>::respect_copy_memory_budget(
           if (accounted_tiles.count(id) == 0) {
             accounted_tiles.emplace(id);
 
+            // Skip for fields added in schema evolution.
+            if (!fragment_metadata_[rt->frag_idx()]->array_schema()->is_field(
+                    name)) {
+              continue;
+            }
+
             // Size of the tile in memory.
             auto&& [st, tile_size] =
                 get_attribute_tile_size(name, rt->frag_idx(), rt->tile_idx());
@@ -1432,14 +1499,21 @@ SparseGlobalOrderReader<BitmapType>::respect_copy_memory_budget(
       });
   RETURN_NOT_OK_ELSE_TUPLE(status, logger_->status(status), nullopt);
 
-  if (max_cs_idx == 0)
+  if (max_cs_idx == 0) {
     return {Status_SparseUnorderedWithDupsReaderError(
                 "Unable to copy one slab with current budget/buffers"),
             nullopt};
+  }
 
   // Resize the result tiles vector.
   buffers_full_ &= max_cs_idx == result_cell_slabs.size();
-  result_cell_slabs.resize(max_cs_idx);
+  while (result_cell_slabs.size() > max_cs_idx) {
+    // Revert progress for this slab in read state, and pop it.
+    auto& last_rcs = result_cell_slabs.back();
+    read_state_.frag_idx_[last_rcs.tile_->frag_idx()] =
+        FragIdx(last_rcs.tile_->tile_idx(), last_rcs.start_);
+    result_cell_slabs.pop_back();
+  }
 
   return {Status::Ok(), std::move(total_mem_usage_per_attr)};
 }
@@ -1589,7 +1663,7 @@ Status SparseGlobalOrderReader<BitmapType>::process_slabs(
       auto& query_buffer = buffers_[name];
 
       // Pointers to var size data, generated when offsets are processed.
-      std::vector<void*> var_data;
+      std::vector<const void*> var_data;
       if (var_sized) {
         var_data.resize(
             cell_offsets[result_cell_slabs.size()] - cell_offsets[0]);
