@@ -394,8 +394,8 @@ Status SparseIndexReaderBase::compute_tile_bitmaps(
   const auto dim_num = array_schema_.dim_num();
   const auto cell_order = array_schema_.cell_order();
 
-  // No subarray set, return.
-  if (!subarray_.is_set()) {
+  // No subarray set or empty result tiles, return.
+  if (!subarray_.is_set() || result_tiles.empty()) {
     return Status::Ok();
   }
 
