@@ -509,46 +509,85 @@ Status SparseUnorderedWithDupsReader<uint64_t>::copy_offsets_tile(
     uint64_t src_max_pos,
     OffType* buffer,
     uint8_t* val_buffer,
-    void** var_data) {
+    const void** var_data) {
   // Get source buffers.
   const auto tile_tuple = rt->tile_tuple(name);
-  const auto t = &std::get<0>(*tile_tuple);
-  const auto t_var = &std::get<1>(*tile_tuple);
-  const auto src_buff = t->data_as<uint64_t>();
-  const auto src_var_buff = t_var->data_as<char>();
-  const auto t_val = &std::get<2>(*tile_tuple);
   const auto cell_num =
       fragment_metadata_[rt->frag_idx()]->cell_num(rt->tile_idx());
 
-  // Process all cells. Last cell might be taken out for vectorization.
-  uint64_t end = (src_max_pos == cell_num) ? src_max_pos - 1 : src_max_pos;
-  for (uint64_t c = src_min_pos; c < end; c++) {
-    for (uint64_t i = 0; i < rt->bitmap_[c]; i++) {
-      *buffer = (OffType)(src_buff[c + 1] - src_buff[c]) / offset_div;
-      buffer++;
-      *var_data = src_var_buff + src_buff[c];
-      var_data++;
-    }
+  // If the tile_tuple is null, this is a field added in schema
+  // evolution. Use the fill value.
+  const uint64_t* src_buff = nullptr;
+  const uint8_t* src_var_buff = nullptr;
+  bool use_fill_value = false;
+  OffType fill_value_size = 0;
+  uint64_t t_var_size = 0;
+  if (tile_tuple == nullptr) {
+    use_fill_value = true;
+    fill_value_size = static_cast<OffType>(
+        array_schema_.attribute(name)->fill_value().size());
+    src_var_buff = array_schema_.attribute(name)->fill_value().data();
+  } else {
+    const auto t = &std::get<0>(*tile_tuple);
+    const auto t_var = &std::get<1>(*tile_tuple);
+    t_var_size = t_var->size();
+    src_buff = t->template data_as<uint64_t>();
+    src_var_buff = t_var->template data_as<uint8_t>();
   }
 
-  // Do last cell.
-  if (src_max_pos == cell_num) {
-    for (uint64_t i = 0; i < rt->bitmap_[src_max_pos - 1]; i++) {
-      *buffer =
-          (OffType)(t_var->size() - src_buff[src_max_pos - 1]) / offset_div;
-      buffer++;
-      *var_data = src_var_buff + src_buff[src_max_pos - 1];
-      var_data++;
+  // Process all cells. Last cell might be taken out for vectorization.
+  uint64_t end = (src_max_pos == cell_num && !use_fill_value) ?
+                     src_max_pos - 1 :
+                     src_max_pos;
+  if (!use_fill_value) {
+    for (uint64_t c = src_min_pos; c < end; c++) {
+      for (uint64_t i = 0; i < rt->bitmap_[c]; i++) {
+        *buffer = (OffType)(src_buff[c + 1] - src_buff[c]) / offset_div;
+        buffer++;
+        *var_data = src_var_buff + src_buff[c];
+        var_data++;
+      }
+    }
+
+    // Do last cell.
+    if (src_max_pos == cell_num) {
+      for (uint64_t i = 0; i < rt->bitmap_[src_max_pos - 1]; i++) {
+        *buffer =
+            (OffType)(t_var_size - src_buff[src_max_pos - 1]) / offset_div;
+        buffer++;
+        *var_data = src_var_buff + src_buff[src_max_pos - 1];
+        var_data++;
+      }
+    }
+  } else {
+    for (uint64_t c = src_min_pos; c < end; c++) {
+      for (uint64_t i = 0; i < rt->bitmap_[c]; i++) {
+        *buffer = fill_value_size / offset_div;
+        buffer++;
+        *var_data = src_var_buff;
+        var_data++;
+      }
     }
   }
 
   // Copy nullable values.
   if (nullable) {
-    const auto src_val_buff = t_val->data_as<uint8_t>();
-    for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
-      for (uint64_t i = 0; i < rt->bitmap_[c]; i++) {
-        *val_buffer = src_val_buff[c];
-        val_buffer++;
+    if (!use_fill_value) {
+      const auto t_val = &std::get<2>(*tile_tuple);
+      const auto src_val_buff = t_val->data_as<uint8_t>();
+      for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
+        for (uint64_t i = 0; i < rt->bitmap_[c]; i++) {
+          *val_buffer = src_val_buff[c];
+          val_buffer++;
+        }
+      }
+    } else {
+      uint8_t v = array_schema_.attribute(name)->fill_value_validity();
+      for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
+        for (uint64_t i = 0; i < rt->bitmap_[c]; i++) {
+          *val_buffer = v;
+          val_buffer++;
+        }
       }
     }
   }
@@ -568,44 +607,82 @@ Status SparseUnorderedWithDupsReader<uint8_t>::copy_offsets_tile(
     const uint64_t src_max_pos,
     OffType* buffer,
     uint8_t* val_buffer,
-    void** var_data) {
+    const void** var_data) {
   // Get source buffers.
-  const auto tile_tuple = rt->tile_tuple(name);
-  const auto t = &std::get<0>(*tile_tuple);
-  const auto t_var = &std::get<1>(*tile_tuple);
-  const auto src_buff = t->data_as<uint64_t>();
-  const auto src_var_buff = t_var->data_as<char>();
-  const auto t_val = &std::get<2>(*tile_tuple);
   const auto cell_num =
       fragment_metadata_[rt->frag_idx()]->cell_num(rt->tile_idx());
+  const auto tile_tuple = rt->tile_tuple(name);
 
-  // 0 sized bitmap means full tile, full tile copy done below.
-  if (rt->has_bmp()) {
+  // If the tile_tuple is null, this is a field added in schema
+  // evolution. Use the fill value.
+  const uint64_t* src_buff = nullptr;
+  const uint8_t* src_var_buff = nullptr;
+  bool use_fill_value = false;
+  OffType fill_value_size = 0;
+  uint64_t t_var_size = 0;
+  if (tile_tuple == nullptr) {
+    use_fill_value = true;
+    fill_value_size = static_cast<OffType>(
+        array_schema_.attribute(name)->fill_value().size());
+    src_var_buff = array_schema_.attribute(name)->fill_value().data();
+  } else {
+    const auto t = &std::get<0>(*tile_tuple);
+    const auto t_var = &std::get<1>(*tile_tuple);
+    t_var_size = t_var->size();
+    src_buff = t->template data_as<uint64_t>();
+    src_var_buff = t_var->template data_as<uint8_t>();
+  }
+
+  if (rt->has_bmp() || use_fill_value) {
     // Process all cells. Last cell might be taken out for vectorization.
-    uint64_t end = (src_max_pos == cell_num) ? src_max_pos - 1 : src_max_pos;
-    for (uint64_t c = src_min_pos; c < end; c++) {
-      if (rt->bitmap_[c]) {
-        *buffer = (OffType)(src_buff[c + 1] - src_buff[c]) / offset_div;
-        buffer++;
-        *var_data = src_var_buff + src_buff[c];
-        var_data++;
+    uint64_t end = (src_max_pos == cell_num && !use_fill_value) ?
+                       src_max_pos - 1 :
+                       src_max_pos;
+    if (!use_fill_value) {
+      for (uint64_t c = src_min_pos; c < end; c++) {
+        if (rt->bitmap_[c]) {
+          *buffer = (OffType)(src_buff[c + 1] - src_buff[c]) / offset_div;
+          buffer++;
+          *var_data = src_var_buff + src_buff[c];
+          var_data++;
+        }
       }
-    }
 
-    // Do last cell.
-    if (src_max_pos == cell_num && rt->bitmap_[src_max_pos - 1]) {
-      *buffer =
-          (OffType)(t_var->size() - src_buff[src_max_pos - 1]) / offset_div;
-      *var_data = src_var_buff + src_buff[src_max_pos - 1];
+      // Do last cell.
+      if (src_max_pos == cell_num && rt->bitmap_[src_max_pos - 1]) {
+        *buffer =
+            (OffType)(t_var_size - src_buff[src_max_pos - 1]) / offset_div;
+        *var_data = src_var_buff + src_buff[src_max_pos - 1];
+      }
+    } else {
+      for (uint64_t c = src_min_pos; c < end; c++) {
+        if (!rt->has_bmp() || rt->bitmap_[c]) {
+          *buffer = fill_value_size / offset_div;
+          buffer++;
+          *var_data = src_var_buff;
+          var_data++;
+        }
+      }
     }
 
     // Copy nullable values.
     if (nullable) {
-      const auto src_val_buff = t_val->data_as<uint8_t>();
-      for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
-        if (rt->bitmap_[c]) {
-          *val_buffer = src_val_buff[c];
-          val_buffer++;
+      if (!use_fill_value) {
+        const auto t_val = &std::get<2>(*tile_tuple);
+        const auto src_val_buff = t_val->data_as<uint8_t>();
+        for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
+          if (rt->bitmap_[c]) {
+            *val_buffer = src_val_buff[c];
+            val_buffer++;
+          }
+        }
+      } else {
+        uint8_t v = array_schema_.attribute(name)->fill_value_validity();
+        for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
+          if (rt->bitmap_[c]) {
+            *val_buffer = v;
+            val_buffer++;
+          }
         }
       }
     }
@@ -621,14 +698,14 @@ Status SparseUnorderedWithDupsReader<uint8_t>::copy_offsets_tile(
 
     // Copy last cell.
     if (src_max_pos == cell_num) {
-      *buffer =
-          (OffType)(t_var->size() - src_buff[src_max_pos - 1]) / offset_div;
+      *buffer = (OffType)(t_var_size - src_buff[src_max_pos - 1]) / offset_div;
       *var_data = src_var_buff + src_buff[src_max_pos - 1];
     }
 
     // Copy nullable values.
     if (nullable) {
-      const auto src_val_buff = t_val->data_as<uint8_t>();
+      const auto t_val = &std::get<2>(*tile_tuple);
+      const auto src_val_buff = t_val->template data_as<uint8_t>();
       for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
         *val_buffer = src_val_buff[c];
         val_buffer++;
@@ -649,7 +726,7 @@ Status SparseUnorderedWithDupsReader<BitmapType>::copy_offsets_tiles(
     const std::vector<ResultTile*>& result_tiles,
     const std::vector<uint64_t>& cell_offsets,
     QueryBuffer& query_buffer,
-    std::vector<void*>& var_data) {
+    std::vector<const void*>& var_data) {
   auto timer_se = stats_->start_timer("copy_offsets_tiles");
 
   // For easy reference.
@@ -767,7 +844,7 @@ Status SparseUnorderedWithDupsReader<BitmapType>::copy_var_data_tiles(
     const std::vector<ResultTile*>& result_tiles,
     const std::vector<uint64_t>& cell_offsets,
     QueryBuffer& query_buffer,
-    std::vector<void*>& var_data) {
+    std::vector<const void*>& var_data) {
   auto timer_se = stats_->start_timer("copy_var_tiles");
 
   // For easy reference.
@@ -834,15 +911,34 @@ Status SparseUnorderedWithDupsReader<uint64_t>::copy_fixed_data_tile(
   const auto tile_tuple = stores_zipped_coords ?
                               rt->tile_tuple(constants::coords) :
                               rt->tile_tuple(name);
-  const auto t = &std::get<0>(*tile_tuple);
-  const auto src_buff = t->data_as<uint8_t>();
+
+  // If the tile_tuple is null, this is a field added in schema
+  // evolution. Use the fill value.
+  const uint8_t* src_buff = nullptr;
+  bool use_fill_value = false;
+  if (tile_tuple == nullptr) {
+    use_fill_value = true;
+    src_buff = array_schema_.attribute(name)->fill_value().data();
+  } else {
+    const auto& t = std::get<0>(*tile_tuple);
+    src_buff = t.template data_as<uint8_t>();
+  }
 
   // Copy values.
   if (!stores_zipped_coords) {
-    for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
-      for (uint64_t i = 0; i < rt->bitmap_[c]; i++) {
-        memcpy(buffer, src_buff + c * cell_size, cell_size);
-        buffer += cell_size;
+    if (!use_fill_value) {
+      for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
+        for (uint64_t i = 0; i < rt->bitmap_[c]; i++) {
+          memcpy(buffer, src_buff + c * cell_size, cell_size);
+          buffer += cell_size;
+        }
+      }
+    } else {
+      for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
+        for (uint64_t i = 0; i < rt->bitmap_[c]; i++) {
+          memcpy(buffer, src_buff, cell_size);
+          buffer += cell_size;
+        }
       }
     }
   } else {  // Copy for zipped coords.
@@ -858,12 +954,22 @@ Status SparseUnorderedWithDupsReader<uint64_t>::copy_fixed_data_tile(
 
   // Copy nullable values.
   if (nullable) {
-    const auto t_val = &std::get<2>(*tile_tuple);
-    const auto src_val_buff = t_val->data_as<uint8_t>();
-    for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
-      for (uint64_t i = 0; i < rt->bitmap_[c]; i++) {
-        *val_buffer = src_val_buff[c];
-        val_buffer++;
+    if (!use_fill_value) {
+      const auto t_val = &std::get<2>(*tile_tuple);
+      const auto src_val_buff = t_val->data_as<uint8_t>();
+      for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
+        for (uint64_t i = 0; i < rt->bitmap_[c]; i++) {
+          *val_buffer = src_val_buff[c];
+          val_buffer++;
+        }
+      }
+    } else {
+      uint8_t v = array_schema_.attribute(name)->fill_value_validity();
+      for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
+        for (uint64_t i = 0; i < rt->bitmap_[c]; i++) {
+          *val_buffer = v;
+          val_buffer++;
+        }
       }
     }
   }
@@ -889,36 +995,53 @@ Status SparseUnorderedWithDupsReader<uint8_t>::copy_fixed_data_tile(
   const auto tile_tuple = stores_zipped_coords ?
                               rt->tile_tuple(constants::coords) :
                               rt->tile_tuple(name);
-  const auto t = &std::get<0>(*tile_tuple);
-  const auto src_buff = t->data_as<uint8_t>();
-  const auto t_val = &std::get<2>(*tile_tuple);
 
-  // Partial tile copy, full tile copy done below.
-  if (rt->has_bmp()) {
+  // If the tile_tuple is null, this is a field added in schema
+  // evolution. Use the fill value.
+  const uint8_t* src_buff = nullptr;
+  bool use_fill_value = false;
+  if (tile_tuple == nullptr) {
+    use_fill_value = true;
+    src_buff = array_schema_.attribute(name)->fill_value().data();
+  } else {
+    const auto& t = std::get<0>(*tile_tuple);
+    src_buff = t.template data_as<uint8_t>();
+  }
+
+  if (rt->has_bmp() || use_fill_value) {
     // Copy values.
     if (!stores_zipped_coords) {
-      // Go through bitmap, when there is a hole in cell contiguity, do a
-      // memcpy.
-      uint64_t length = 0;
-      uint64_t start = src_min_pos;
-      for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
-        if (rt->bitmap_[c]) {
-          length++;
-        } else {
-          if (length != 0) {
-            memcpy(buffer, src_buff + start * cell_size, length * cell_size);
-            buffer += length * cell_size;
-            length = 0;
+      if (!use_fill_value) {
+        // Go through bitmap, when there is a hole in cell contiguity, do a
+        // memcpy.
+        uint64_t length = 0;
+        uint64_t start = src_min_pos;
+        for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
+          if (rt->bitmap_[c]) {
+            length++;
+          } else {
+            if (length != 0) {
+              memcpy(buffer, src_buff + start * cell_size, length * cell_size);
+              buffer += length * cell_size;
+              length = 0;
+            }
+
+            start = c + 1;
           }
-
-          start = c + 1;
         }
-      }
 
-      // Do last memcpy.
-      if (length != 0) {
-        memcpy(buffer, src_buff + start * cell_size, length * cell_size);
-        buffer += length * cell_size;
+        // Do last memcpy.
+        if (length != 0) {
+          memcpy(buffer, src_buff + start * cell_size, length * cell_size);
+          buffer += length * cell_size;
+        }
+      } else {
+        for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
+          if (!rt->has_bmp() || rt->bitmap_[c]) {
+            memcpy(buffer, src_buff, cell_size);
+            buffer += cell_size;
+          }
+        }
       }
     } else {  // Copy for zipped coords.
       const auto dim_num = rt->domain()->dim_num();
@@ -933,29 +1056,40 @@ Status SparseUnorderedWithDupsReader<uint8_t>::copy_fixed_data_tile(
 
     // Copy nullable values.
     if (nullable) {
-      // Go through bitmap, when there is a hole in cell contiguity, do a
-      // memcpy.
-      uint64_t length = 0;
-      uint64_t start = src_min_pos;
-      const auto src_val_buff = t_val->data_as<uint8_t>();
-      for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
-        if (rt->bitmap_[c]) {
-          length++;
-        } else {
-          if (length != 0) {
-            memcpy(val_buffer, src_val_buff + start, length);
-            val_buffer += length;
-            length = 0;
+      if (!use_fill_value) {
+        // Go through bitmap, when there is a hole in cell contiguity, do a
+        // memcpy.
+        const auto t_val = &std::get<2>(*tile_tuple);
+        uint64_t length = 0;
+        uint64_t start = src_min_pos;
+        const auto src_val_buff = t_val->data_as<uint8_t>();
+        for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
+          if (rt->bitmap_[c]) {
+            length++;
+          } else {
+            if (length != 0) {
+              memcpy(val_buffer, src_val_buff + start, length);
+              val_buffer += length;
+              length = 0;
+            }
+
+            start = c + 1;
           }
-
-          start = c + 1;
         }
-      }
 
-      // Do last memcpy.
-      if (length != 0) {
-        memcpy(val_buffer, src_val_buff + start, length);
-        val_buffer += length;
+        // Do last memcpy.
+        if (length != 0) {
+          memcpy(val_buffer, src_val_buff + start, length);
+          val_buffer += length;
+        }
+      } else {
+        uint8_t v = array_schema_.attribute(name)->fill_value_validity();
+        for (uint64_t c = src_min_pos; c < src_max_pos; c++) {
+          for (uint64_t i = 0; i < rt->bitmap_[c]; i++) {
+            *val_buffer = v;
+            val_buffer++;
+          }
+        }
       }
     }
   } else {  // Copy full tile.
@@ -965,7 +1099,8 @@ Status SparseUnorderedWithDupsReader<uint8_t>::copy_fixed_data_tile(
         (src_max_pos - src_min_pos) * cell_size);
 
     if (nullable) {
-      const auto src_val_buff = t_val->data_as<uint8_t>();
+      const auto t_val = &std::get<2>(*tile_tuple);
+      const auto src_val_buff = t_val->template data_as<uint8_t>();
       memcpy(val_buffer, src_val_buff + src_min_pos, src_max_pos - src_min_pos);
     }
   }
@@ -1284,6 +1419,12 @@ SparseUnorderedWithDupsReader<BitmapType>::respect_copy_memory_budget(
           // Size of the tile in memory.
           auto rt = (UnorderedWithDupsResultTile<BitmapType>*)result_tiles[idx];
 
+          // Skip for fields added in schema evolution.
+          if (!fragment_metadata_[rt->frag_idx()]->array_schema()->is_field(
+                  name)) {
+            continue;
+          }
+
           auto&& [st, tile_size] =
               get_attribute_tile_size(name, rt->frag_idx(), rt->tile_idx());
           RETURN_NOT_OK(st);
@@ -1462,7 +1603,7 @@ Status SparseUnorderedWithDupsReader<BitmapType>::process_tiles(
       }
 
       // Pointers to var size data, generated when offsets are processed.
-      std::vector<void*> var_data;
+      std::vector<const void*> var_data;
       if (var_sized) {
         var_data.resize(cell_offsets[result_tiles.size()] - cell_offsets[0]);
       }
