@@ -265,6 +265,52 @@ Status RestClient::post_array_schema_to_rest(
   return sc;
 }
 
+Status RestClient::post_array_from_rest(const URI& uri, Array* array) {
+  if (array == nullptr) {
+    return LOG_STATUS(Status_SerializationError(
+        "Error getting remote array; array is null."));
+  }
+
+  Buffer buff;
+  RETURN_NOT_OK(
+      serialization::array_open_serialize(*array, serialization_type_, &buff));
+  // Wrap in a list
+  BufferList serialized;
+  RETURN_NOT_OK(serialized.add_buffer(std::move(buff)));
+
+  // Init curl and form the URL
+  Curl curlc(logger_);
+  std::string array_ns, array_uri;
+  RETURN_NOT_OK(uri.get_rest_components(&array_ns, &array_uri));
+  const std::string cache_key = array_ns + ":" + array_uri;
+  RETURN_NOT_OK(
+      curlc.init(config_, extra_headers_, &redirect_meta_, &redirect_mtx_));
+  std::string url = redirect_uri(cache_key) + "/v2/arrays/" + array_ns + "/" +
+                    curlc.url_escape(array_uri) + "/?";
+
+  // Remote array operations should provide start and end timestamps
+  url += "start_timestamp=" + std::to_string(array->timestamp_start()) +
+         "&end_timestamp=" + std::to_string(array->timestamp_end());
+
+  // Get the data
+  Buffer returned_data;
+  RETURN_NOT_OK(curlc.post_data(
+      stats_,
+      url,
+      serialization_type_,
+      &serialized,
+      &returned_data,
+      cache_key));
+  if (returned_data.data() == nullptr || returned_data.size() == 0)
+    return LOG_STATUS(Status_RestError(
+        "Error getting array from REST; server returned no data."));
+
+  // Ensure data has a null delimiter for cap'n proto if using JSON
+  RETURN_NOT_OK(ensure_json_null_delimited_string(&returned_data));
+  return serialization::array_deserialize(
+      array, serialization_type_, returned_data);
+}
+
 Status RestClient::deregister_array_from_rest(const URI& uri) {
   // Init curl and form the URL
   Curl curlc(logger_);
@@ -1139,6 +1185,11 @@ Status RestClient::post_array_schema_to_rest(const URI&, const ArraySchema&) {
       Status_RestError("Cannot use rest client; serialization not enabled."));
 }
 
+Status RestClient::post_array_from_rest(const URI&, Array*) {
+  return LOG_STATUS(
+      Status_RestError("Cannot use rest client; serialization not enabled."));
+}
+
 Status RestClient::deregister_array_from_rest(const URI&) {
   return LOG_STATUS(
       Status_RestError("Cannot use rest client; serialization not enabled."));
